/*
	file automatically generated by make_test_files.pl
	Wed Mar 12 18:04:49 2003
*/

/*****************************************************************************
 *                                                                           *
 *          UNURAN -- Universal Non-Uniform Random number generator          *
 *                                                                           *
 *****************************************************************************/
    
/**
 ** Tests for CORDER
 **/
    
/*---------------------------------------------------------------------------*/
#include "testunuran.h"
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global variables                                                          */

static FILE *TESTLOG;               /* test log file                         */
static FILE *UNURANLOG;             /* unuran log file                       */

static int test_ok = TRUE;          /* all tests ok (boolean)                */
static int n_tests_failed;          /* number of failed tests                */

/*---------------------------------------------------------------------------*/

void test_new( void );
void test_set( void );
void test_get( void );
void test_chg( void );
void test_init( void );
void test_reinit( void );
void test_sample( void );
void test_validate( void );

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par );

int unur_corder_set_verify( UNUR_PAR *par, int );

/*---------------------------------------------------------------------------*/


/* prototypes */

#define COMPARE_SAMPLE_SIZE   (500)
#define VIOLATE_SAMPLE_SIZE   (20)





/*---------------------------------------------------------------------------*/

int main()
{ 
	/* open log file for unuran and set output stream for unuran messages */
	UNURANLOG = fopen( "t_corder_unuran.log","w" );
	abort_if_NULL( stderr,-1, UNURANLOG );
	unur_set_stream( UNURANLOG );

	/* open log file for testing */
	TESTLOG = fopen( "t_corder_test.log","w" );
	abort_if_NULL( stderr,-1, TESTLOG );

	/* write header into log file */
  	{
		time_t started;  
		fprintf(TESTLOG,"\nUNURAN - Universal Non-Uniform RANdom number generator\n\n");
		if (time( &started ) != -1)
			fprintf(TESTLOG,"%s",ctime(&started));
		fprintf(TESTLOG,"\n====================================================\n\n");
	}

        /* seed build-in uniform generators */
        unur_urng_MRG31k3p_seed(625380);
	unur_urng_fish_seed(625380);
	unur_urng_mstd_seed(625380);

	/* seed uniform random number generator */
#if UNUR_URNG_TYPE == UNUR_URNG_PRNG
	unur_set_default_urng(prng_new("mt19937(625380)"));
#elif UNUR_URNG_TYPE == UNUR_URNG_RNGSTREAM
	{
	    unsigned long seed[6] = {625380,625380+1,625380+2,625380+3,625380+4,625380+5};
	    RngStream_SetPackageSeed(seed);
	}
#endif  /* UNUR_URNG_TYPE */

	/* set default debugging flag */
	unur_set_default_debug(UNUR_DEBUG_ALL);

	/* start test */
	printf("corder: ");

	/* run tests */
	test_new();
	test_set();
	test_get();
	test_chg();
	test_init();
	test_sample();
	test_reinit();
	test_validate();

	/* test finished */
	printf("\n");  fflush(stdout);

	/* close log files */
	fprintf(TESTLOG,"\n====================================================\n\n");
	if (test_ok)
		fprintf(TESTLOG,"All tests PASSED.\n");
	else
		fprintf(TESTLOG,"Test(s) FAILED.\n");

	fclose(UNURANLOG);
	fclose(TESTLOG);

	/* free memory */
        compare_free_memory();

	/* exit */
	exit( (test_ok) ? 0 : -1 );

} /* end of main */

/*---------------------------------------------------------------------------*/

void test_new (void)
{
	/* start test */
	printf("[new "); fflush(stdout);
	fprintf(TESTLOG,"\n[new]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 27, (unur_distr_corder_new( distr, 5, 2 )) );
n_tests_failed += check_errorcode( TESTLOG, 27, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 33, (unur_distr_corder_new( distr, 5, 2 )) );
n_tests_failed += check_errorcode( TESTLOG, 33, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* invalid order statistics */
{
UNUR_DISTR *distr = NULL;
   UNUR_DISTR *os;
   distr = unur_distr_normal(NULL,0);
   os = unur_distr_corder_new( distr, 5, 2 ); 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 41, (unur_distr_corder_new( os, 5, 2 )) );
n_tests_failed += check_errorcode( TESTLOG, 41, UNUR_ERR_DISTR_INVALID );
unur_distr_free(os);
unur_distr_free(distr);
}

/* invalid parameter */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_normal(NULL,0); 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 49, (unur_distr_corder_new( distr, 5, 10 )) );
n_tests_failed += check_errorcode( TESTLOG, 49, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 52, (unur_distr_corder_new( distr, 1, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 52, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = new  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_new() */

/*---------------------------------------------------------------------------*/

void test_set (void)
{
	/* start test */
	printf("[set "); fflush(stdout);
	fprintf(TESTLOG,"\n[set]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 63, (unur_distr_corder_set_rank( distr, 5, 2 )) );
n_tests_failed += check_errorcode( TESTLOG, 63, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 66, (unur_distr_corder_set_pdfparams( distr, NULL, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 66, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 69, (unur_distr_corder_set_domain( distr, 0., 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 69, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 72, (unur_distr_corder_set_mode( distr, 0. )) );
n_tests_failed += check_errorcode( TESTLOG, 72, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 75, (unur_distr_corder_set_pdfarea( distr, 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 75, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid distribution object */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_normal(NULL,0); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 82, (unur_distr_corder_set_rank( distr, 5, 2 )) );
n_tests_failed += check_errorcode( TESTLOG, 82, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* invalid distribution object */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 88, (unur_distr_corder_set_pdfparams( distr, NULL, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 88, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 91, (unur_distr_corder_set_domain( distr, 0., 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 91, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 94, (unur_distr_corder_set_mode( distr, 0. )) );
n_tests_failed += check_errorcode( TESTLOG, 94, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 97, (unur_distr_corder_set_pdfarea( distr, 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 97, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   UNUR_DISTR *os;
   distr = unur_distr_normal(NULL,0);
   os = unur_distr_corder_new( distr, 5, 2 ); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 105, (unur_distr_corder_set_pdfparams( distr, NULL, 1 )) );
n_tests_failed += check_errorcode( TESTLOG, 105, UNUR_ERR_NULL );
unur_distr_free(os);
unur_distr_free(distr);
}

/* invalid parameters */
{
UNUR_DISTR *distr = NULL;
   UNUR_DISTR *os;
   distr = unur_distr_normal(NULL,0);
   os = unur_distr_corder_new( distr, 5, 2 ); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 115, (unur_distr_corder_set_rank( os, 5, 20 )) );
n_tests_failed += check_errorcode( TESTLOG, 115, UNUR_ERR_DISTR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 118, (unur_distr_cont_set_pdfarea( distr, -1. )) );
n_tests_failed += check_errorcode( TESTLOG, 118, UNUR_ERR_DISTR_SET );
unur_distr_free(os);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = set  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_set() */

/*---------------------------------------------------------------------------*/

void test_get (void)
{
	/* start test */
	printf("[get "); fflush(stdout);
	fprintf(TESTLOG,"\n[get]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid distribution object */
{
UNUR_DISTR *distr = NULL;
   int n, k;
   distr = unur_distr_normal(NULL,0); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 131, ((int) unur_distr_corder_get_distribution( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 131, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 134, (unur_distr_corder_get_rank( distr, &n, &k )) );
n_tests_failed += check_errorcode( TESTLOG, 134, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = get  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_get() */

/*---------------------------------------------------------------------------*/

void test_chg (void)
{
	/* start test */
	printf("[chg "); fflush(stdout);
	fprintf(TESTLOG,"\n[chg]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = chg  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_chg() */

/*---------------------------------------------------------------------------*/

void test_init (void)
{
	/* start test */
	printf("[init "); fflush(stdout);
	fprintf(TESTLOG,"\n[init]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = init  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_init() */

/*---------------------------------------------------------------------------*/

void test_reinit (void)
{
	/* start test */
	printf("[reinit "); fflush(stdout);
	fprintf(TESTLOG,"\n[reinit]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = reinit  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_reinit() */

/*---------------------------------------------------------------------------*/

void test_sample (void)
{
	/* start test */
	printf("[sample "); fflush(stdout);
	fprintf(TESTLOG,"\n[sample]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* compare clone */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   UNUR_GEN *clone;
   UNUR_DISTR *d;
   d = unur_distr_cauchy(NULL,0);
   distr = unur_distr_corder_new( d, 10, 8);
   unur_distr_free(d);
   par = unur_arou_new(distr);
   unur_arou_set_max_sqhratio(par,0.);
   gen = unur_init( par ); 
abort_if_NULL(TESTLOG, 160,    gen );

unur_errno = 0;
/* original generator object */;
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 163, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
/* clone */
clone = unur_gen_clone(gen);
unur_free(gen);
gen = clone;
n_tests_failed += compare_double_sequence_gen( TESTLOG, 169, gen, COMPARE_SAMPLE_SIZE );
unur_free(gen);
unur_distr_free(distr);
}

/* compare stringparser */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   UNUR_DISTR *os = NULL;
   distr = NULL;
   par = NULL;
   gen = NULL; 

unur_errno = 0;
distr = unur_distr_normal(NULL,0);
unur_distr_cont_set_domain(distr,0,UNUR_INFINITY);
os = unur_distr_corder_new(distr,10,7);
par = unur_arou_new(os);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 184, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
unur_distr_free(os); os = NULL;
gen = unur_str2gen("normal; domain=(0.,inf); orderstatistics=(10,7) & \
		    method = arou");
n_tests_failed += compare_double_sequence_gen( TESTLOG, 191, gen, COMPARE_SAMPLE_SIZE );
unur_free(gen);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = sample  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_sample() */

/*---------------------------------------------------------------------------*/

void test_validate (void)
{
	UNUR_DISTR *distr[16];
	UNUR_PAR *par;
	UNUR_GEN *gen;
	int n_tests_failed = 0;
	double *darray;
	double fpm[10];

	/* start test */
	printf("[validate "); fflush(stdout);
	fprintf(TESTLOG,"\n[validate]\n");

	/* reset counter */
	n_tests_failed = 0;


/* distributions: 16 */
{ UNUR_DISTR *d;
fpm[0] = 1.;
fpm[1] = 2.;
d = unur_distr_beta(fpm,2);
distr[0] = unur_distr_corder_new( d, 10, 8);
unur_distr_free(d); }


{ UNUR_DISTR *d;
fpm[0] = 3.;
fpm[1] = 4.;
d = unur_distr_beta(fpm,2);
distr[1] = unur_distr_corder_new( d, 10, 8);
unur_distr_free(d); }


{ UNUR_DISTR *d;
fpm[0] = 5.;
fpm[1] = 100.;
d = unur_distr_beta(fpm,2);
distr[2] = unur_distr_corder_new( d, 10, 8);
unur_distr_free(d); }


{ UNUR_DISTR *d;
d = unur_distr_cauchy(NULL,0);
distr[3] = unur_distr_corder_new( d, 10, 8);
unur_distr_free(d); }


{ UNUR_DISTR *d;
fpm[0] = 1.;
d = unur_distr_gamma( fpm,1 );
distr[4] = unur_distr_corder_new( d, 10, 8);
unur_distr_free(d); }


{ UNUR_DISTR *d;
fpm[0] = 2.;
d = unur_distr_gamma( fpm,1 );
distr[5] = unur_distr_corder_new( d, 10, 8);
unur_distr_free(d); }


{ UNUR_DISTR *d;
fpm[0] = 3.;
d = unur_distr_gamma( fpm,1 );
distr[6] = unur_distr_corder_new( d, 10, 8);
unur_distr_free(d); }


{ UNUR_DISTR *d;
fpm[0] = 10.;
d = unur_distr_gamma( fpm,1 );
distr[7] = unur_distr_corder_new( d, 10, 8);
unur_distr_free(d); }


{ UNUR_DISTR *d;
fpm[0] = 100.;
d = unur_distr_gamma( fpm,1 );
distr[8] = unur_distr_corder_new( d, 10, 8);
unur_distr_free(d); }


{ UNUR_DISTR *d;
d = unur_distr_normal( NULL,0 );
distr[9] = unur_distr_corder_new( d, 10, 8);
unur_distr_free(d); }


{ UNUR_DISTR *d;
fpm[0] = 1.;
fpm[1] = 0.01;
d = unur_distr_normal( fpm,2 );
distr[10] = unur_distr_corder_new( d, 10, 8);
unur_distr_free(d); }


{ UNUR_DISTR *d;
fpm[0] = 0.;
fpm[1] = 100.;
d = unur_distr_normal( fpm,2 );
distr[11] = unur_distr_corder_new( d, 10, 8);
unur_distr_free(d); }


{ UNUR_DISTR *d;
d = unur_distr_uniform(NULL,0);
distr[12] = unur_distr_corder_new( d, 10, 8);
unur_distr_free(d); }


{ UNUR_DISTR *d;
d = unur_distr_cauchy(NULL,0);
distr[13] = unur_distr_corder_new( d, 10, 8);
unur_distr_cont_set_domain(distr[13],0.1,1.);
unur_distr_cont_upd_pdfarea(distr[13]);
unur_distr_free(d); }


{UNUR_DISTR *d;
d = unur_distr_cauchy(NULL,0);
unur_distr_cont_set_domain(d,0.1,1.);
unur_distr_cont_upd_mode(d);
unur_distr_cont_upd_pdfarea(d);
distr[14] = unur_distr_corder_new( d, 10, 8);
unur_distr_free(d); }


{UNUR_DISTR *d;
d = unur_str2distr("cont; pdf=\"exp(-x)\"; cdf=\"1-exp(-x)\"; domain=(0,inf); mode=0; pdfarea=1;");
distr[15] = unur_distr_corder_new( d, 10, 8);
unur_distr_free(d); }


	printf("\n(chi^2) "); fflush(stdout);

/* chi^2 tests: 48 */

	unur_set_default_debug(~UNUR_DEBUG_SAMPLE);
	fprintf( TESTLOG,"\nChi^2 Test:\n");

/* distribution [0] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[0]);
unur_tdr_set_cpoints(par,50,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_arou_new(distr[0]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ninv_new(distr[0]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [1] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[1]);
unur_tdr_set_cpoints(par,50,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_arou_new(distr[1]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ninv_new(distr[1]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [2] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[2]);
unur_tdr_set_cpoints(par,50,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_arou_new(distr[2]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ninv_new(distr[2]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [3] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[3]);
unur_tdr_set_cpoints(par,50,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_arou_new(distr[3]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ninv_new(distr[3]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [4] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[4]);
unur_tdr_set_cpoints(par,50,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_arou_new(distr[4]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ninv_new(distr[4]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [5] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[5]);
unur_tdr_set_cpoints(par,50,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_arou_new(distr[5]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ninv_new(distr[5]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [6] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[6]);
unur_tdr_set_cpoints(par,50,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_arou_new(distr[6]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ninv_new(distr[6]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [7] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[7]);
unur_tdr_set_cpoints(par,50,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_arou_new(distr[7]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ninv_new(distr[7]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [8] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[8]);
unur_tdr_set_cpoints(par,50,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_arou_new(distr[8]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ninv_new(distr[8]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [9] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[9]);
unur_tdr_set_cpoints(par,50,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_arou_new(distr[9]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ninv_new(distr[9]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [10] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[10]);
unur_tdr_set_cpoints(par,50,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_arou_new(distr[10]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ninv_new(distr[10]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [11] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[11]);
unur_tdr_set_cpoints(par,50,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_arou_new(distr[11]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ninv_new(distr[11]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [12] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[12]);
unur_tdr_set_cpoints(par,50,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_arou_new(distr[12]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ninv_new(distr[12]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [13] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[13]);
unur_tdr_set_cpoints(par,50,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_arou_new(distr[13]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ninv_new(distr[13]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [14] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[14]);
unur_tdr_set_cpoints(par,50,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_arou_new(distr[14]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ninv_new(distr[14]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [15] */

	do {
	unur_errno = 0;
par = unur_tdr_new(distr[15]);
unur_tdr_set_cpoints(par,50,NULL);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_arou_new(distr[15]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ninv_new(distr[15]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);


/* free distributions */
	unur_distr_free(distr[0]);
	unur_distr_free(distr[1]);
	unur_distr_free(distr[2]);
	unur_distr_free(distr[3]);
	unur_distr_free(distr[4]);
	unur_distr_free(distr[5]);
	unur_distr_free(distr[6]);
	unur_distr_free(distr[7]);
	unur_distr_free(distr[8]);
	unur_distr_free(distr[9]);
	unur_distr_free(distr[10]);
	unur_distr_free(distr[11]);
	unur_distr_free(distr[12]);
	unur_distr_free(distr[13]);
	unur_distr_free(distr[14]);
	unur_distr_free(distr[15]);
#if WITH_DMALLOC
	dmalloc_vmessage("section = validate  ================================\n","");
	dmalloc_log_unfreed();
#endif


	/* test finished */
	test_ok &= (n_tests_failed>2) ? 0 : 1;   /* we accept 2 failure */
	(n_tests_failed>2) ? printf(" --> failed] ") : printf(" --> ok] ");

	/* prevent compiler from making useless annoying warnings */
	distr[0] = NULL;
	par = NULL;
	gen = NULL;
	darray = NULL;
	fpm[0] = 0.;

} /* end of test_validate */

/*---------------------------------------------------------------------------*/




/*---------------------------------------------------------------------------*/
/* run generator in verifying mode */

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par )
{
	UNUR_GEN *gen;
	int i;

	/* switch to verifying mode */
	unur_corder_set_verify(par,1);

	/* initialize generator */
	gen = unur_init( par ); abort_if_NULL(LOG, line, gen);

	/* run generator */
	for (i=0; i<VIOLATE_SAMPLE_SIZE; i++)
		unur_sample_cont(gen);

	/* destroy generator */
	unur_free(gen); 

} /* end of run_verify_generator() */

int unur_corder_set_verify(UNUR_PAR *par, int verify) {return 0;}
