/*
	file automatically generated by make_test_files.pl
	Wed Mar 12 18:04:50 2003
*/

/*****************************************************************************
 *                                                                           *
 *          UNURAN -- Universal Non-Uniform Random number generator          *
 *                                                                           *
 *****************************************************************************/
    
/**
 ** Tests for DISTR_CVEC
 **/
    
/*---------------------------------------------------------------------------*/
#include "testunuran.h"
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global variables                                                          */

static FILE *TESTLOG;               /* test log file                         */
static FILE *UNURANLOG;             /* unuran log file                       */

static int test_ok = TRUE;          /* all tests ok (boolean)                */
static int n_tests_failed;          /* number of failed tests                */

/*---------------------------------------------------------------------------*/

void test_new( void );
void test_set( void );
void test_get( void );
void test_chg( void );
void test_init( void );
void test_reinit( void );
void test_sample( void );
void test_validate( void );

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par );

int unur_distr_cvec_set_verify( UNUR_PAR *par, int );

/*---------------------------------------------------------------------------*/


/* prototypes */

#define COMPARE_SAMPLE_SIZE  (500)
#define VIOLATE_SAMPLE_SIZE   (20)

double mvpdf(const double *x, const UNUR_DISTR *distr);
int mvdpdf(double *result, const double *x, const UNUR_DISTR *distr);





/*---------------------------------------------------------------------------*/

int main()
{ 
	/* open log file for unuran and set output stream for unuran messages */
	UNURANLOG = fopen( "t_distr_cvec_unuran.log","w" );
	abort_if_NULL( stderr,-1, UNURANLOG );
	unur_set_stream( UNURANLOG );

	/* open log file for testing */
	TESTLOG = fopen( "t_distr_cvec_test.log","w" );
	abort_if_NULL( stderr,-1, TESTLOG );

	/* write header into log file */
  	{
		time_t started;  
		fprintf(TESTLOG,"\nUNURAN - Universal Non-Uniform RANdom number generator\n\n");
		if (time( &started ) != -1)
			fprintf(TESTLOG,"%s",ctime(&started));
		fprintf(TESTLOG,"\n====================================================\n\n");
	}

        /* seed build-in uniform generators */
        unur_urng_MRG31k3p_seed(654066);
	unur_urng_fish_seed(654066);
	unur_urng_mstd_seed(654066);

	/* seed uniform random number generator */
#if UNUR_URNG_TYPE == UNUR_URNG_PRNG
	unur_set_default_urng(prng_new("mt19937(654066)"));
#elif UNUR_URNG_TYPE == UNUR_URNG_RNGSTREAM
	{
	    unsigned long seed[6] = {654066,654066+1,654066+2,654066+3,654066+4,654066+5};
	    RngStream_SetPackageSeed(seed);
	}
#endif  /* UNUR_URNG_TYPE */

	/* set default debugging flag */
	unur_set_default_debug(UNUR_DEBUG_ALL);

	/* start test */
	printf("distr_cvec: ");

	/* run tests */
	test_new();
	test_set();
	test_get();
	test_chg();
	test_init();
	test_sample();
	test_reinit();
	test_validate();

	/* test finished */
	printf("\n");  fflush(stdout);

	/* close log files */
	fprintf(TESTLOG,"\n====================================================\n\n");
	if (test_ok)
		fprintf(TESTLOG,"All tests PASSED.\n");
	else
		fprintf(TESTLOG,"Test(s) FAILED.\n");

	fclose(UNURANLOG);
	fclose(TESTLOG);

	/* free memory */
        compare_free_memory();

	/* exit */
	exit( (test_ok) ? 0 : -1 );

} /* end of main */

/*---------------------------------------------------------------------------*/

void test_new (void)
{
	/* start test */
	printf("[new "); fflush(stdout);
	fprintf(TESTLOG,"\n[new]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid data */
{
UNUR_DISTR *distr = NULL;
	distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 30, (unur_distr_cvec_new(1)) );
n_tests_failed += check_errorcode( TESTLOG, 30, UNUR_ERR_DISTR_SET );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = new  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_new() */

/*---------------------------------------------------------------------------*/

void test_set (void)
{
	/* start test */
	printf("[set "); fflush(stdout);
	fprintf(TESTLOG,"\n[set]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 41, (unur_distr_cvec_set_pdf( distr, mvpdf )) );
n_tests_failed += check_errorcode( TESTLOG, 41, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 44, (unur_distr_cvec_set_dpdf( distr, mvdpdf )) );
n_tests_failed += check_errorcode( TESTLOG, 44, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 47, (unur_distr_cvec_set_mean( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 47, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 50, (unur_distr_cvec_set_covar( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 50, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 53, (unur_distr_cvec_set_mode( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 53, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 56, (unur_distr_cvec_set_pdfparams( distr, 0, NULL, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 56, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 59, (unur_distr_cvec_set_pdfvol( distr, 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 59, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   double fpm[2] = {1.,2.};
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 67, (unur_distr_cvec_set_pdf( distr, mvpdf )) );
n_tests_failed += check_errorcode( TESTLOG, 67, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 70, (unur_distr_cvec_set_dpdf( distr, mvdpdf )) );
n_tests_failed += check_errorcode( TESTLOG, 70, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 73, (unur_distr_cvec_set_mean( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 73, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 76, (unur_distr_cvec_set_covar( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 76, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 79, (unur_distr_cvec_set_mode( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 79, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 82, (unur_distr_cvec_set_pdfparams( distr, 0, fpm, 2 )) );
n_tests_failed += check_errorcode( TESTLOG, 82, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 85, (unur_distr_cvec_set_pdfvol( distr, 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 85, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cvec_new(2); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 92, (unur_distr_cvec_set_pdf( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 92, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 95, (unur_distr_cvec_set_dpdf( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 95, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 98, (unur_distr_cvec_set_pdfparams( distr, 0, NULL, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 98, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid parameters */
{
UNUR_DISTR *distr = NULL;
   double fpm[2] = {1.,2.};
   double covar_invalid_negative[4] = {-1.,0.,0.,1.};
   double covar_invalid_nonsym[4] = {1.,0.5,0.,1.};
   distr = unur_distr_cvec_new(2); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 108, (unur_distr_cvec_set_pdfvol( distr, -1. )) );
n_tests_failed += check_errorcode( TESTLOG, 108, UNUR_ERR_DISTR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 111, (unur_distr_cvec_set_covar( distr, covar_invalid_negative )) );
n_tests_failed += check_errorcode( TESTLOG, 111, UNUR_ERR_DISTR_DOMAIN );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 114, (unur_distr_cvec_set_covar( distr, covar_invalid_nonsym )) );
n_tests_failed += check_errorcode( TESTLOG, 114, UNUR_ERR_DISTR_DOMAIN );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 117, (unur_distr_cvec_set_mode( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 117, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 120, (unur_distr_cvec_set_pdfparams( distr, -1, fpm, 2 )) );
n_tests_failed += check_errorcode( TESTLOG, 120, UNUR_ERR_DISTR_NPARAMS );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 123, (unur_distr_cvec_set_pdfparams( distr, 100000, fpm, 2 )) );
n_tests_failed += check_errorcode( TESTLOG, 123, UNUR_ERR_DISTR_NPARAMS );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = set  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_set() */

/*---------------------------------------------------------------------------*/

void test_get (void)
{
	/* start test */
	printf("[get "); fflush(stdout);
	fprintf(TESTLOG,"\n[get]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 134, ((void *) unur_distr_cvec_get_pdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 134, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 137, ((void *) unur_distr_cvec_get_dpdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 137, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 140, (unur_distr_cvec_get_pdfvol( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 140, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 143, (unur_distr_cvec_get_mean( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 143, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 146, (unur_distr_cvec_get_covar( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 146, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 149, (unur_distr_cvec_get_mode( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 149, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 152, (unur_distr_cvec_get_pdfparams( distr, 0, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 152, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   const double *fpm;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 160, ((void *) unur_distr_cvec_get_pdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 160, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 163, ((void *) unur_distr_cvec_get_dpdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 163, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 166, (unur_distr_cvec_get_pdfvol( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 166, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 169, (unur_distr_cvec_get_mean( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 169, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 172, (unur_distr_cvec_get_covar( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 172, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 175, (unur_distr_cvec_get_mode( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 175, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 178, (unur_distr_cvec_get_pdfparams( distr, 0, &fpm )) );
n_tests_failed += check_errorcode( TESTLOG, 178, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* parameters not unknown */
{
UNUR_DISTR *distr = NULL;
   const double *fpm;
   distr = unur_distr_cvec_new(2); 

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 186, (unur_distr_cvec_get_pdfvol( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 186, UNUR_ERR_DISTR_GET );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 189, (unur_distr_cvec_get_mean( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 189, UNUR_ERR_DISTR_GET );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 192, (unur_distr_cvec_get_covar( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 192, UNUR_ERR_DISTR_GET );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 195, (unur_distr_cvec_get_mode( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 195, UNUR_ERR_DISTR_GET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 198, (unur_distr_cvec_get_pdfparams( distr, -1, &fpm )) );
n_tests_failed += check_errorcode( TESTLOG, 198, UNUR_ERR_DISTR_NPARAMS );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 201, (unur_distr_cvec_get_pdfparams( distr, 1000, &fpm )) );
n_tests_failed += check_errorcode( TESTLOG, 201, UNUR_ERR_DISTR_NPARAMS );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = get  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_get() */

/*---------------------------------------------------------------------------*/

void test_chg (void)
{
	/* start test */
	printf("[chg "); fflush(stdout);
	fprintf(TESTLOG,"\n[chg]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = chg  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_chg() */

/*---------------------------------------------------------------------------*/

void test_init (void)
{
	/* start test */
	printf("[init "); fflush(stdout);
	fprintf(TESTLOG,"\n[init]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = init  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_init() */

/*---------------------------------------------------------------------------*/

void test_reinit (void)
{
	/* start test */
	printf("[reinit "); fflush(stdout);
	fprintf(TESTLOG,"\n[reinit]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = reinit  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_reinit() */

/*---------------------------------------------------------------------------*/

void test_sample (void)
{
	/* start test */
	printf("[sample "); fflush(stdout);
	fprintf(TESTLOG,"\n[sample]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 224, (unur_distr_cvec_eval_pdf( NULL, distr )) );
n_tests_failed += check_errorcode( TESTLOG, 224, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 227, (unur_distr_cvec_eval_dpdf( NULL, NULL, distr )) );
n_tests_failed += check_errorcode( TESTLOG, 227, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid distribution object */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 234, (unur_distr_cvec_eval_pdf( NULL, distr )) );
n_tests_failed += check_errorcode( TESTLOG, 234, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 237, (unur_distr_cvec_eval_dpdf( NULL, NULL, distr )) );
n_tests_failed += check_errorcode( TESTLOG, 237, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* missing data */
{
UNUR_DISTR *distr = NULL;
   double x[2], result[2];
   distr = unur_distr_cvec_new(2); 

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 245, (unur_distr_cvec_eval_pdf( x, distr )) );
n_tests_failed += check_errorcode( TESTLOG, 245, UNUR_ERR_DISTR_DATA );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 248, (unur_distr_cvec_eval_dpdf( result, x, distr )) );
n_tests_failed += check_errorcode( TESTLOG, 248, UNUR_ERR_DISTR_DATA );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = sample  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_sample() */

/*---------------------------------------------------------------------------*/

void test_validate (void)
{
	UNUR_DISTR *distr[1];
	UNUR_PAR *par;
	UNUR_GEN *gen;
	int n_tests_failed = 0;
	double *darray;
	double fpm[10];

	/* start test */
	printf("[validate "); fflush(stdout);
	fprintf(TESTLOG,"\n[validate]\n");

	/* reset counter */
	n_tests_failed = 0;


/* distributions: 0 */

/* free distributions */
#if WITH_DMALLOC
	dmalloc_vmessage("section = validate  ================================\n","");
	dmalloc_log_unfreed();
#endif


	/* test finished */
	test_ok &= (n_tests_failed>2) ? 0 : 1;   /* we accept 2 failure */
	(n_tests_failed>2) ? printf(" --> failed] ") : printf(" --> ok] ");

	/* prevent compiler from making useless annoying warnings */
	distr[0] = NULL;
	par = NULL;
	gen = NULL;
	darray = NULL;
	fpm[0] = 0.;

} /* end of test_validate */

/*---------------------------------------------------------------------------*/


double mvpdf(const double *x, const UNUR_DISTR *distr) { return 1.; }
int mvdpdf(double *result, const double *x, const UNUR_DISTR *distr) { return 1; }



/*---------------------------------------------------------------------------*/
/* run generator in verifying mode */

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par )
{
	UNUR_GEN *gen;
	int i;

	/* switch to verifying mode */
	unur_distr_cvec_set_verify(par,1);

	/* initialize generator */
	gen = unur_init( par ); abort_if_NULL(LOG, line, gen);

	/* run generator */
	for (i=0; i<VIOLATE_SAMPLE_SIZE; i++)
		unur_sample_cont(gen);

	/* destroy generator */
	unur_free(gen); 

} /* end of run_verify_generator() */

int unur_distr_cvec_set_verify(UNUR_PAR *par, int verify) {return 0;}
