/*
	file automatically generated by make_test_files.pl
	Wed Mar 12 18:04:51 2003
*/

/*****************************************************************************
 *                                                                           *
 *          UNURAN -- Universal Non-Uniform Random number generator          *
 *                                                                           *
 *****************************************************************************/
    
/**
 ** Tests for HINV
 **/
    
/*---------------------------------------------------------------------------*/
#include "testunuran.h"
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global variables                                                          */

static FILE *TESTLOG;               /* test log file                         */
static FILE *UNURANLOG;             /* unuran log file                       */

static int test_ok = TRUE;          /* all tests ok (boolean)                */
static int n_tests_failed;          /* number of failed tests                */

/*---------------------------------------------------------------------------*/

void test_new( void );
void test_set( void );
void test_get( void );
void test_chg( void );
void test_init( void );
void test_reinit( void );
void test_sample( void );
void test_validate( void );

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par );

int unur_hinv_set_verify( UNUR_PAR *par, int );

/*---------------------------------------------------------------------------*/


/* prototypes */

#define COMPARE_SAMPLE_SIZE   (500)
#define VIOLATE_SAMPLE_SIZE   (20)

#ifndef M_PI
#define M_PI       3.14159265358979323846264338328      /* pi */
#endif

double sin_cdf( double x, const UNUR_DISTR *distr );
double sin_pdf( double x, const UNUR_DISTR *distr );
double sin_dpdf( double x, const UNUR_DISTR *distr );

double sin0_cdf( double x, const UNUR_DISTR *distr );
double sin0_pdf( double x, const UNUR_DISTR *distr );
double sin0_dpdf( double x, const UNUR_DISTR *distr );





/*---------------------------------------------------------------------------*/

int main()
{ 
	/* open log file for unuran and set output stream for unuran messages */
	UNURANLOG = fopen( "t_hinv_unuran.log","w" );
	abort_if_NULL( stderr,-1, UNURANLOG );
	unur_set_stream( UNURANLOG );

	/* open log file for testing */
	TESTLOG = fopen( "t_hinv_test.log","w" );
	abort_if_NULL( stderr,-1, TESTLOG );

	/* write header into log file */
  	{
		time_t started;  
		fprintf(TESTLOG,"\nUNURAN - Universal Non-Uniform RANdom number generator\n\n");
		if (time( &started ) != -1)
			fprintf(TESTLOG,"%s",ctime(&started));
		fprintf(TESTLOG,"\n====================================================\n\n");
	}

        /* seed build-in uniform generators */
        unur_urng_MRG31k3p_seed(933841);
	unur_urng_fish_seed(933841);
	unur_urng_mstd_seed(933841);

	/* seed uniform random number generator */
#if UNUR_URNG_TYPE == UNUR_URNG_PRNG
	unur_set_default_urng(prng_new("mt19937(933841)"));
#elif UNUR_URNG_TYPE == UNUR_URNG_RNGSTREAM
	{
	    unsigned long seed[6] = {933841,933841+1,933841+2,933841+3,933841+4,933841+5};
	    RngStream_SetPackageSeed(seed);
	}
#endif  /* UNUR_URNG_TYPE */

	/* set default debugging flag */
	unur_set_default_debug(UNUR_DEBUG_ALL);

	/* start test */
	printf("hinv: ");

	/* run tests */
	test_new();
	test_set();
	test_get();
	test_chg();
	test_init();
	test_sample();
	test_reinit();
	test_validate();

	/* test finished */
	printf("\n");  fflush(stdout);

	/* close log files */
	fprintf(TESTLOG,"\n====================================================\n\n");
	if (test_ok)
		fprintf(TESTLOG,"All tests PASSED.\n");
	else
		fprintf(TESTLOG,"Test(s) FAILED.\n");

	fclose(UNURANLOG);
	fclose(TESTLOG);

	/* free memory */
        compare_free_memory();

	/* exit */
	exit( (test_ok) ? 0 : -1 );

} /* end of main */

/*---------------------------------------------------------------------------*/

void test_new (void)
{
	/* start test */
	printf("[new "); fflush(stdout);
	fprintf(TESTLOG,"\n[new]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 38, (unur_hinv_new( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 38, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 44, (unur_hinv_new( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 44, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* data missing in distribution object */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
/* cdf */
n_tests_failed += check_expected_NULL( TESTLOG, 51, (unur_hinv_new( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 51, UNUR_ERR_DISTR_REQUIRED );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = new  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_new() */

/*---------------------------------------------------------------------------*/

void test_set (void)
{
	/* start test */
	printf("[set "); fflush(stdout);
	fprintf(TESTLOG,"\n[set]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_PAR   *par = NULL;
   double stp[] = {1.,2.,3.};
   par = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 62, (unur_hinv_set_order( par, 3 )) );
n_tests_failed += check_errorcode( TESTLOG, 62, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 65, (unur_hinv_set_u_resolution( par, 1.e-8 )) );
n_tests_failed += check_errorcode( TESTLOG, 65, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 68, (unur_hinv_set_cpoints(par,stp,3)) );
n_tests_failed += check_errorcode( TESTLOG, 68, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 71, (unur_hinv_set_boundary(par,1.,3.)) );
n_tests_failed += check_errorcode( TESTLOG, 71, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 74, (unur_hinv_set_guidefactor( par, 2. )) );
n_tests_failed += check_errorcode( TESTLOG, 74, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 77, (unur_hinv_set_max_intervals(par,10000.)) );
n_tests_failed += check_errorcode( TESTLOG, 77, UNUR_ERR_NULL );
}

/* invalid parameter object */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   double fpar[2] = {0.,1.};
   double stp[] = {1.,2.,3.};
   distr = unur_distr_normal(fpar,2);
   par = unur_arou_new(distr); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 86, (unur_hinv_set_order( par, 3 )) );
n_tests_failed += check_errorcode( TESTLOG, 86, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 89, (unur_hinv_set_u_resolution( par, 1.e-8 )) );
n_tests_failed += check_errorcode( TESTLOG, 89, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 92, (unur_hinv_set_cpoints(par,stp,3)) );
n_tests_failed += check_errorcode( TESTLOG, 92, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 95, (unur_hinv_set_boundary(par,1.,3.)) );
n_tests_failed += check_errorcode( TESTLOG, 95, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 98, (unur_hinv_set_guidefactor( par, 2. )) );
n_tests_failed += check_errorcode( TESTLOG, 98, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 101, (unur_hinv_set_max_intervals(par,10000)) );
n_tests_failed += check_errorcode( TESTLOG, 101, UNUR_ERR_PAR_INVALID );
free(par);
unur_distr_free(distr);
}

/* invalid parameters */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   double fpar[2] = {0.,1.};
   double stp[] = {1.,2.,-3.};
   distr = unur_distr_normal(fpar,2);
   par = unur_hinv_new(distr); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 112, (unur_hinv_set_order( par, 30 )) );
n_tests_failed += check_errorcode( TESTLOG, 112, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 115, (unur_hinv_set_u_resolution( par, -1. )) );
n_tests_failed += check_errorcode( TESTLOG, 115, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 118, (unur_hinv_set_cpoints(par,stp,3)) );
n_tests_failed += check_errorcode( TESTLOG, 118, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 121, (unur_hinv_set_boundary(par,1.,-3.)) );
n_tests_failed += check_errorcode( TESTLOG, 121, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 124, (unur_hinv_set_boundary(par,1.,UNUR_INFINITY)) );
n_tests_failed += check_errorcode( TESTLOG, 124, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 127, (unur_hinv_set_guidefactor( par, -2. )) );
n_tests_failed += check_errorcode( TESTLOG, 127, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 130, (unur_hinv_set_max_intervals(par,0)) );
n_tests_failed += check_errorcode( TESTLOG, 130, UNUR_ERR_PAR_SET );
free(par);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = set  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_set() */

/*---------------------------------------------------------------------------*/

void test_get (void)
{
	/* start test */
	printf("[get "); fflush(stdout);
	fprintf(TESTLOG,"\n[get]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid generator object */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   distr = unur_distr_normal(NULL,0);
   par = unur_tdr_new(distr);
   gen = unur_init(par); 
abort_if_NULL(TESTLOG, 141,    gen );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 144, (unur_hinv_get_n_intervals(gen)) );
n_tests_failed += check_errorcode( TESTLOG, 144, UNUR_ERR_GEN_INVALID );
unur_free(gen);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = get  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_get() */

/*---------------------------------------------------------------------------*/

void test_chg (void)
{
	/* start test */
	printf("[chg "); fflush(stdout);
	fprintf(TESTLOG,"\n[chg]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid generator object */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[2] = {0.,1.};
   distr = unur_distr_normal(fpar,2);
   unur_distr_cont_set_domain(distr,-1.,2.);
   par = unur_hinv_new(distr);
   unur_set_debug(par,0);
   gen = unur_init( par ); 
abort_if_NULL(TESTLOG, 157,    gen );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 160, (unur_hinv_chg_truncated( gen, 1., 0. )) );
n_tests_failed += check_errorcode( TESTLOG, 160, UNUR_ERR_DISTR_SET );

unur_errno = 0;
unur_hinv_chg_truncated( gen, -2., 0. );
n_tests_failed += check_errorcode( TESTLOG, 163, UNUR_ERR_DISTR_SET );

unur_errno = 0;
unur_hinv_chg_truncated( gen, 0., 10. );
n_tests_failed += check_errorcode( TESTLOG, 166, UNUR_ERR_DISTR_SET );
unur_free(gen);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = chg  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_chg() */

/*---------------------------------------------------------------------------*/

void test_init (void)
{
	/* start test */
	printf("[init "); fflush(stdout);
	fprintf(TESTLOG,"\n[init]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = init  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_init() */

/*---------------------------------------------------------------------------*/

void test_reinit (void)
{
	/* start test */
	printf("[reinit "); fflush(stdout);
	fprintf(TESTLOG,"\n[reinit]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = reinit  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_reinit() */

/*---------------------------------------------------------------------------*/

void test_sample (void)
{
	/* start test */
	printf("[sample "); fflush(stdout);
	fprintf(TESTLOG,"\n[sample]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_GEN   *gen = NULL;
   gen = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 185, (unur_hinv_eval_approxinvcdf(gen,0.5)) );
n_tests_failed += check_errorcode( TESTLOG, 185, UNUR_ERR_NULL );
unur_free(gen);
}

/* invalid generator object */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   distr = unur_distr_normal(NULL,0);
   par = unur_tdr_new(distr);
   gen = unur_init(par); 
abort_if_NULL(TESTLOG, 192,    gen );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 195, (unur_hinv_eval_approxinvcdf(gen,0.5)) );
n_tests_failed += check_errorcode( TESTLOG, 195, UNUR_ERR_GEN_INVALID );
unur_free(gen);
unur_distr_free(distr);
}

/* invalid domain */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   distr = unur_distr_normal(NULL,0);
   par = unur_hinv_new(distr);
   unur_set_debug(par,0);
   gen = unur_init( par ); 
abort_if_NULL(TESTLOG, 202,    gen );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 205, (unur_hinv_eval_approxinvcdf(gen,1.5)) );
n_tests_failed += check_errorcode( TESTLOG, 205, UNUR_ERR_DOMAIN );

unur_errno = 0;
n_tests_failed += check_expected_negINFINITY( TESTLOG, 208, (unur_hinv_eval_approxinvcdf(gen,-0.5)) );
n_tests_failed += check_errorcode( TESTLOG, 208, UNUR_ERR_DOMAIN );
unur_free(gen);
unur_distr_free(distr);
}

/* compare clone */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   UNUR_GEN *clone;
   distr = unur_distr_normal(NULL,0); 
   par = NULL;
   gen = NULL; 

unur_errno = 0;
/* original generator object */
par = unur_hinv_new(distr);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 221, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
/* clone */
clone = unur_gen_clone(gen);
unur_free(gen);
gen = clone;
n_tests_failed += compare_double_sequence_gen( TESTLOG, 227, gen, COMPARE_SAMPLE_SIZE );
unur_free(gen);
unur_distr_free(distr);
}

/* compare stringparser */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[] = {3.,4.};
   distr = NULL;
   par = NULL;
   gen = NULL; 

unur_errno = 0;
distr = unur_distr_gamma(fpar,2);
unur_distr_cont_set_domain(distr,2,UNUR_INFINITY);
par = unur_hinv_new(distr);
unur_set_debug(par,1u);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 243, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "gamma(3.,4.); domain = 2,infinity & \
  method = hinv; debug = 0x1" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 249, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
distr = unur_distr_normal(NULL,0);
unur_distr_cont_set_domain(distr,2,UNUR_INFINITY);
par = unur_hinv_new(distr);
unur_hinv_set_order(par,1);
unur_hinv_set_max_intervals(par,1000);
unur_hinv_set_u_resolution(par,0.001);
unur_hinv_set_boundary(par,2,5);
unur_set_debug(par,1u);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 262, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "normal; domain = (2,inf) & \
         method = hinv; order = 1; max_intervals = 1000; u_resolution = 0.001; \
         boundary = (2,5); debug = 0x1" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 269, gen, COMPARE_SAMPLE_SIZE );
unur_free(gen);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = sample  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_sample() */

/*---------------------------------------------------------------------------*/

void test_validate (void)
{
	UNUR_DISTR *distr[35];
	UNUR_PAR *par;
	UNUR_GEN *gen;
	int n_tests_failed = 0;
	double *darray;
	double fpm[10];

	/* start test */
	printf("[validate "); fflush(stdout);
	fprintf(TESTLOG,"\n[validate]\n");

	/* reset counter */
	n_tests_failed = 0;


/* distributions: 35 */
fpm[0] = 1.;
fpm[1] = 2.;
distr[0] = unur_distr_beta(fpm,2);


fpm[0] = 1.;
fpm[1] = 5.;
distr[1] = unur_distr_beta(fpm,2);


fpm[0] = 1.;
fpm[1] = 100.;
distr[2] = unur_distr_beta(fpm,2);


fpm[0] = 3.;
fpm[1] = 4.;
distr[3] = unur_distr_beta(fpm,2);


fpm[0] = 5.;
fpm[1] = 100.;
distr[4] = unur_distr_beta(fpm,2);


fpm[0] = 500.;
fpm[1] = 300.;
distr[5] = unur_distr_beta(fpm,2);


fpm[0] = 5.;
fpm[1] = 10.;
fpm[2] = -3.;
fpm[3] = 15.;
distr[6] = unur_distr_beta(fpm,4);


fpm[0] = 0.5;
fpm[1] = 3.;
distr[28] = unur_distr_beta(fpm,2);


fpm[0] = 5.;
fpm[1] = 0.3;
distr[29] = unur_distr_beta(fpm,2);


fpm[0] = 0.5;
fpm[1] = 0.4;
distr[30] = unur_distr_beta(fpm,2);


distr[7] = unur_distr_cauchy(NULL,0);


fpm[0] = 1.;
fpm[1] = 20.;
distr[8] = unur_distr_cauchy(fpm,2);


distr[24] = unur_distr_exponential(NULL,0);


fpm[0] = 30.;
fpm[1] = -5.;
distr[25] = unur_distr_exponential(fpm,2);


fpm[0] = 1.;
distr[9] = unur_distr_gamma(fpm,1);


fpm[0] = 2.;
distr[10] = unur_distr_gamma(fpm,1);


fpm[0] = 3.;
distr[11] = unur_distr_gamma(fpm,1);


fpm[0] = 10.;
distr[12] = unur_distr_gamma(fpm,1);


fpm[0] = 1000.;
distr[13] = unur_distr_gamma(fpm,1);


fpm[0] = 5.;
fpm[1] = 1000.;
distr[14] = unur_distr_gamma(fpm,2);


fpm[0] = 5.;
fpm[1] = 1.e-5;
distr[15] = unur_distr_gamma(fpm,2);


fpm[0] = 5.;
fpm[1] = 10.;
fpm[2] = 1000;
distr[16] = unur_distr_gamma(fpm,3);


fpm[0] = 0.5;
distr[31] = unur_distr_gamma(fpm,1);


fpm[0] = 0.2;
distr[32] = unur_distr_gamma(fpm,1);


distr[26] = unur_distr_laplace(NULL,0);


fpm[0] = -10.;
fpm[1] = 100.;
distr[27] = unur_distr_laplace(fpm,2);


distr[17] = unur_distr_normal(NULL,0);


fpm[0] = 1.;
fpm[1] = 1.e-5;
distr[18] = unur_distr_normal(fpm,2);


fpm[0] = 0.;
fpm[1] = 1.e+5;
distr[19] = unur_distr_normal(fpm,2);


distr[20] = unur_distr_uniform(NULL,0);


fpm[0] = 1.;
fpm[1] = 20.;
distr[21] = unur_distr_uniform(fpm,2);


distr[22] = unur_distr_cauchy(NULL,0);
unur_distr_cont_set_domain(distr[22],0.1,1.);
unur_distr_cont_upd_mode(distr[22]);
unur_distr_cont_upd_pdfarea(distr[22]);


fpm[0] = 3.;
fpm[1] = 4.;
distr[23] = unur_distr_beta(fpm,2);
unur_distr_cont_set_domain(distr[23],-2.,5.);


distr[33] = unur_distr_cont_new();
unur_distr_cont_set_cdf( distr[33], sin_cdf );
unur_distr_cont_set_pdf( distr[33], sin_pdf );
unur_distr_cont_set_dpdf( distr[33], sin_dpdf );
unur_distr_cont_set_domain( distr[33], -1., 1. );


distr[34] = unur_distr_cont_new();
unur_distr_cont_set_cdf( distr[34], sin0_cdf );
unur_distr_cont_set_pdf( distr[34], sin0_pdf );
unur_distr_cont_set_dpdf( distr[34], sin0_dpdf );
unur_distr_cont_set_domain( distr[34], -1., 1. );


	printf("\n(chi^2) "); fflush(stdout);

/* chi^2 tests: 245 */

	unur_set_default_debug(~UNUR_DEBUG_SAMPLE);
	fprintf( TESTLOG,"\nChi^2 Test:\n");

/* distribution [0] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[0]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[0]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[0]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[0]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[0]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[0]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[0]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [1] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[1]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[1]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[1]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[1]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[1]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[1]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[1]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [2] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[2]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[2]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[2]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[2]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[2]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[2]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [3] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[3]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[3]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[3]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[3]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[3]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[3]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[3]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [4] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[4]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[4]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[4]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[4]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[4]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[4]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [5] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[5]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[5]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[5]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[5]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[5]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[5]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[5]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [6] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[6]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[6]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[6]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[6]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[6]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[6]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[6]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [28] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[28]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[28]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[28]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[28]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[28]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[28]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[28]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [29] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[29]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[29]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[29]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[29]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[29]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[29]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[29]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [30] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[30]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[30]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[30]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[30]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[30]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[30]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[30]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [7] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[7]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[7]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[7]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[7]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[7]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[7]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[7]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [8] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[8]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[8]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[8]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[8]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[8]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[8]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[8]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [24] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[24]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[24]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[24]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[24]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[24]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[24]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[24]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [25] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[25]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[25]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[25]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[25]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[25]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[25]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[25]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [9] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[9]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[9]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[9]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[9]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[9]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[9]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[9]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [10] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[10]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[10]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[10]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[10]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[10]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[10]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[10]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [11] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[11]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[11]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[11]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[11]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[11]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[11]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[11]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [12] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[12]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[12]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[12]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[12]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[12]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[12]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[12]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [13] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[13]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[13]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[13]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[13]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[13]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[13]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [14] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[14]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[14]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[14]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[14]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[14]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[14]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [15] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[15]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[15]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[15]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[15]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[15]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[15]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [16] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[16]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[16]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[16]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[16]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[16]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[16]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [31] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[31]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[31]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[31]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[31]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[31]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[31]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[31]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [32] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[32]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[32]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[32]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[32]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[32]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[32]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[32]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [26] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[26]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[26]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[26]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[26]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[26]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[26]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[26]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [27] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[27]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[27]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[27]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[27]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[27]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[27]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[27]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [17] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[17]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[17]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[17]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[17]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[17]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[17]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[17]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [18] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[18]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[18]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[18]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[18]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[18]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[18]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [19] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[19]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[19]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[19]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[19]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[19]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[19]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[19]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [20] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[20]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[20]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[20]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[20]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[20]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[20]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[20]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [21] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[21]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[21]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[21]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[21]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[21]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[21]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [22] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[22]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[22]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[22]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[22]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[22]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[22]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[22]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [23] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[23]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[23]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[23]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[23]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[23]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[23]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[23]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [33] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[33]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[33]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[33]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[33]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[33]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[33]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.25;
fpm[2]=  0.25;
fpm[3]=  0.75;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [34] */

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[34]);
unur_hinv_set_order(par,1);
unur_hinv_set_u_resolution(par,1.e-8);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[34]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[34]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[34]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
	gen = unur_init(par);
	if (gen) {
unur_hinv_chg_truncated(gen,0.5,0.9);
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[34]);
unur_hinv_set_order(par,5);
unur_hinv_set_u_resolution(par,1.e-12);
unur_set_debug(par,1u);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_hinv_new(distr[34]);
unur_hinv_set_order(par,3);
unur_hinv_set_u_resolution(par,1.e-10);
unur_set_debug(par,1u);
fpm[0]= -0.75;
fpm[1]= -0.5;
fpm[2]=  0.;
fpm[3]=  0.25;
unur_hinv_set_cpoints(par,fpm,4);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);


/* free distributions */
	unur_distr_free(distr[0]);
	unur_distr_free(distr[1]);
	unur_distr_free(distr[2]);
	unur_distr_free(distr[3]);
	unur_distr_free(distr[4]);
	unur_distr_free(distr[5]);
	unur_distr_free(distr[6]);
	unur_distr_free(distr[7]);
	unur_distr_free(distr[8]);
	unur_distr_free(distr[9]);
	unur_distr_free(distr[10]);
	unur_distr_free(distr[11]);
	unur_distr_free(distr[12]);
	unur_distr_free(distr[13]);
	unur_distr_free(distr[14]);
	unur_distr_free(distr[15]);
	unur_distr_free(distr[16]);
	unur_distr_free(distr[17]);
	unur_distr_free(distr[18]);
	unur_distr_free(distr[19]);
	unur_distr_free(distr[20]);
	unur_distr_free(distr[21]);
	unur_distr_free(distr[22]);
	unur_distr_free(distr[23]);
	unur_distr_free(distr[24]);
	unur_distr_free(distr[25]);
	unur_distr_free(distr[26]);
	unur_distr_free(distr[27]);
	unur_distr_free(distr[28]);
	unur_distr_free(distr[29]);
	unur_distr_free(distr[30]);
	unur_distr_free(distr[31]);
	unur_distr_free(distr[32]);
	unur_distr_free(distr[33]);
	unur_distr_free(distr[34]);
#if WITH_DMALLOC
	dmalloc_vmessage("section = validate  ================================\n","");
	dmalloc_log_unfreed();
#endif


	/* test finished */
	test_ok &= (n_tests_failed>2) ? 0 : 1;   /* we accept 2 failure */
	(n_tests_failed>2) ? printf(" --> failed] ") : printf(" --> ok] ");

	/* prevent compiler from making useless annoying warnings */
	distr[0] = NULL;
	par = NULL;
	gen = NULL;
	darray = NULL;
	fpm[0] = 0.;

} /* end of test_validate */

/*---------------------------------------------------------------------------*/


/* Example of  a distribution with f(x)=0 at x=-0.25 and x=0.75  */
/*                                                               */
/*          /  (1 +sin(2 Pi x))/2  if |x| <= 1                   */
/*  f(x) = <                                                     */
/*          \  0        otherwise                                */
/*                                                               */
double sin_pdf( double x, const UNUR_DISTR *distr )
{
  if (fabs(x) > 1.)
    return 0.;
  else
    return (0.5*(1.+sin((2.*M_PI)*x)));
} /* end of sinpdf() */

/* The derivative of the PDF of our distribution:                */
double sin_dpdf( double x, const UNUR_DISTR *distr )
{
  if (fabs(x) > 1.)
    return 0.;
  else
    return (M_PI*cos((2.*M_PI)*x));
} /* end of sin_dpdf() */

/* The CDF of our distribution:                                  */
double sin_cdf( double x, const UNUR_DISTR *distr )
{
  if (x < -1.)
    return 0.;
  else if(x<=1.)
    return (1.+2.*M_PI*(1+x)-cos((2.*M_PI)*x))/(4.*M_PI);
  else return 1.;
} /* end of sin_cdf() */

/****************************************************************/
/* Example of  a distribution with f(x)=0 in (-0.5,0)            */
/*                                                               */
/*          /  Max(sin(2 Pi x)),0)Pi/2  if -1 < x <0.5           */
/*  f(x) = <                                                     */
/*          \  0        otherwise                                */
/*                                                               */
double sin0_pdf( double x, const UNUR_DISTR *distr )
{
  if (x < -1.) return 0.;
  if (x <= -0.5) return sin((2.*M_PI)*x)*0.5*M_PI;
  if (x < 0.) return 0.;
  if (x <= 0.5) return sin((2.*M_PI)*x)*0.5*M_PI;
  return 0.;
} /* end of sin0_pdf() */

/* The derivative of the PDF of our distribution:                */
double sin0_dpdf( double x, const UNUR_DISTR *distr )
{
  if (x < -1.) return 0.;
  if (x <= -0.5) return cos((2.*M_PI)*x)*M_PI*M_PI;
  if (x < 0.) return 0.;
  if (x <= 0.5) return cos((2.*M_PI)*x)*M_PI*M_PI;
  return 0.;
} /* end of sin0_dpdf() */

/* The CDF of our distribution:                                  */
double sin0_cdf( double x, const UNUR_DISTR *distr )
{
  if (x < -1.) return 0.;
  if(x<= -0.5) return 0.25*(1-cos((2.*M_PI)*x));
  if (x < 0.) return 0.5;
  if (x <= 0.5) return 0.75-0.25*cos((2.*M_PI)*x);
  return 1.;
} /* end of sin0_cdf() */




/*---------------------------------------------------------------------------*/
/* run generator in verifying mode */

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par )
{
	UNUR_GEN *gen;
	int i;

	/* switch to verifying mode */
	unur_hinv_set_verify(par,1);

	/* initialize generator */
	gen = unur_init( par ); abort_if_NULL(LOG, line, gen);

	/* run generator */
	for (i=0; i<VIOLATE_SAMPLE_SIZE; i++)
		unur_sample_cont(gen);

	/* destroy generator */
	unur_free(gen); 

} /* end of run_verify_generator() */

int unur_hinv_set_verify(UNUR_PAR *par, int verify) {return 0;}
