/*
	file automatically generated by make_test_files.pl
	Wed Mar 12 18:04:52 2003
*/

/*****************************************************************************
 *                                                                           *
 *          UNURAN -- Universal Non-Uniform Random number generator          *
 *                                                                           *
 *****************************************************************************/
    
/**
 ** Tests for SSR
 **/
    
/*---------------------------------------------------------------------------*/
#include "testunuran.h"
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global variables                                                          */

static FILE *TESTLOG;               /* test log file                         */
static FILE *UNURANLOG;             /* unuran log file                       */

static int test_ok = TRUE;          /* all tests ok (boolean)                */
static int n_tests_failed;          /* number of failed tests                */

/*---------------------------------------------------------------------------*/

void test_new( void );
void test_set( void );
void test_get( void );
void test_chg( void );
void test_init( void );
void test_reinit( void );
void test_sample( void );
void test_validate( void );

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par );

int unur_ssr_set_verify( UNUR_PAR *par, int );

/*---------------------------------------------------------------------------*/


/* prototypes */
double pdf( double x, const UNUR_DISTR *distr );

double pdf_sqrtlin( double x, const UNUR_DISTR *distr );
double dpdf_sqrtlin( double x, const UNUR_DISTR *distr );
double cdf_sqrtlin( double x, const UNUR_DISTR *distr );

double pdf_sqrtlinshft(double x, const UNUR_DISTR *distr );
double dpdf_sqrtlinshft( double x, const UNUR_DISTR *distr );
double cdf_sqrtlinshft( double x, const UNUR_DISTR *distr );

int unur_ssr_set_pedantic( struct unur_par *par, int pedantic );

#define COMPARE_SAMPLE_SIZE   (10000)
#define VIOLATE_SAMPLE_SIZE   (20)





/*---------------------------------------------------------------------------*/

int main()
{ 
	/* open log file for unuran and set output stream for unuran messages */
	UNURANLOG = fopen( "t_ssr_unuran.log","w" );
	abort_if_NULL( stderr,-1, UNURANLOG );
	unur_set_stream( UNURANLOG );

	/* open log file for testing */
	TESTLOG = fopen( "t_ssr_test.log","w" );
	abort_if_NULL( stderr,-1, TESTLOG );

	/* write header into log file */
  	{
		time_t started;  
		fprintf(TESTLOG,"\nUNURAN - Universal Non-Uniform RANdom number generator\n\n");
		if (time( &started ) != -1)
			fprintf(TESTLOG,"%s",ctime(&started));
		fprintf(TESTLOG,"\n====================================================\n\n");
	}

        /* seed build-in uniform generators */
        unur_urng_MRG31k3p_seed(789894);
	unur_urng_fish_seed(789894);
	unur_urng_mstd_seed(789894);

	/* seed uniform random number generator */
#if UNUR_URNG_TYPE == UNUR_URNG_PRNG
	unur_set_default_urng(prng_new("mt19937(789894)"));
#elif UNUR_URNG_TYPE == UNUR_URNG_RNGSTREAM
	{
	    unsigned long seed[6] = {789894,789894+1,789894+2,789894+3,789894+4,789894+5};
	    RngStream_SetPackageSeed(seed);
	}
#endif  /* UNUR_URNG_TYPE */

	/* set default debugging flag */
	unur_set_default_debug(UNUR_DEBUG_ALL);

	/* start test */
	printf("ssr: ");

	/* run tests */
	test_new();
	test_set();
	test_get();
	test_chg();
	test_init();
	test_sample();
	test_reinit();
	test_validate();

	/* test finished */
	printf("\n");  fflush(stdout);

	/* close log files */
	fprintf(TESTLOG,"\n====================================================\n\n");
	if (test_ok)
		fprintf(TESTLOG,"All tests PASSED.\n");
	else
		fprintf(TESTLOG,"Test(s) FAILED.\n");

	fclose(UNURANLOG);
	fclose(TESTLOG);

	/* free memory */
        compare_free_memory();

	/* exit */
	exit( (test_ok) ? 0 : -1 );

} /* end of main */

/*---------------------------------------------------------------------------*/

void test_new (void)
{
	/* start test */
	printf("[new "); fflush(stdout);
	fprintf(TESTLOG,"\n[new]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 38, (unur_ssr_new( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 38, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 44, (unur_ssr_new( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 44, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* data missing in distribution object */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
/* pdf, mode, pdfarea */
n_tests_failed += check_expected_NULL( TESTLOG, 51, (unur_ssr_new( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 51, UNUR_ERR_DISTR_REQUIRED );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = new  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_new() */

/*---------------------------------------------------------------------------*/

void test_set (void)
{
	/* start test */
	printf("[set "); fflush(stdout);
	fprintf(TESTLOG,"\n[set]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_PAR   *par = NULL;
   par = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 61, (unur_ssr_set_cdfatmode(par,0.)) );
n_tests_failed += check_errorcode( TESTLOG, 61, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 64, (unur_ssr_set_pdfatmode(par,1.)) );
n_tests_failed += check_errorcode( TESTLOG, 64, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 67, (unur_ssr_set_verify(par,1)) );
n_tests_failed += check_errorcode( TESTLOG, 67, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 70, (unur_ssr_set_usesqueeze(par,1)) );
n_tests_failed += check_errorcode( TESTLOG, 70, UNUR_ERR_NULL );
}

/* invalid parameter object */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   double fpar[2] = {0.,1.};
   distr = unur_distr_normal(fpar,2);
   par = unur_arou_new(distr); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 78, (unur_ssr_set_cdfatmode(par,0.)) );
n_tests_failed += check_errorcode( TESTLOG, 78, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 81, (unur_ssr_set_pdfatmode(par,1.)) );
n_tests_failed += check_errorcode( TESTLOG, 81, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 84, (unur_ssr_set_verify(par,1)) );
n_tests_failed += check_errorcode( TESTLOG, 84, UNUR_ERR_PAR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 87, (unur_ssr_set_usesqueeze(par,1)) );
n_tests_failed += check_errorcode( TESTLOG, 87, UNUR_ERR_PAR_INVALID );
free(par);
unur_distr_free(distr);
}

/* invalid parameters */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   double fpar[2] = {0.,1.};
   distr = unur_distr_normal(fpar,2);
   par = unur_ssr_new(distr); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 97, (unur_ssr_set_cdfatmode(par,-1.)) );
n_tests_failed += check_errorcode( TESTLOG, 97, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 100, (unur_ssr_set_pdfatmode(par,-1.)) );
n_tests_failed += check_errorcode( TESTLOG, 100, UNUR_ERR_PAR_SET );
free(par);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = set  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_set() */

/*---------------------------------------------------------------------------*/

void test_get (void)
{
	/* start test */
	printf("[get "); fflush(stdout);
	fprintf(TESTLOG,"\n[get]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = get  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_get() */

/*---------------------------------------------------------------------------*/

void test_chg (void)
{
	/* start test */
	printf("[chg "); fflush(stdout);
	fprintf(TESTLOG,"\n[chg]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid generator object */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[2] = {0.,1.};
   distr = unur_distr_normal(fpar,2);
   par = unur_arou_new(distr);
   unur_set_debug(par,0);
   gen = unur_init( par ); 
abort_if_NULL(TESTLOG, 117,    gen );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 120, (unur_ssr_chg_verify(gen,1)) );
n_tests_failed += check_errorcode( TESTLOG, 120, UNUR_ERR_GEN_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 123, (unur_ssr_chg_pdfparams(gen,fpar,2)) );
n_tests_failed += check_errorcode( TESTLOG, 123, UNUR_ERR_GEN_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 126, (unur_ssr_chg_domain(gen,0.,1.)) );
n_tests_failed += check_errorcode( TESTLOG, 126, UNUR_ERR_GEN_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 129, (unur_ssr_chg_mode(gen,0.)) );
n_tests_failed += check_errorcode( TESTLOG, 129, UNUR_ERR_GEN_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 132, (unur_ssr_chg_cdfatmode(gen,1.)) );
n_tests_failed += check_errorcode( TESTLOG, 132, UNUR_ERR_GEN_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 135, (unur_ssr_chg_pdfatmode(gen,1.)) );
n_tests_failed += check_errorcode( TESTLOG, 135, UNUR_ERR_GEN_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 138, (unur_ssr_chg_pdfarea(gen,1.)) );
n_tests_failed += check_errorcode( TESTLOG, 138, UNUR_ERR_GEN_INVALID );
unur_free(gen);
unur_distr_free(distr);
}

/* invalid parameters */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[2] = {2.,5.};
   distr = unur_distr_gamma(fpar,2);
   par = unur_ssr_new(distr);
   gen = unur_init( par ); 
abort_if_NULL(TESTLOG, 144,    gen );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 147, (unur_ssr_chg_pdfparams(gen,NULL,1)) );
n_tests_failed += check_errorcode( TESTLOG, 147, UNUR_ERR_NULL );

unur_errno = 0;
unur_ssr_chg_pdfparams(gen,fpar,UNUR_DISTR_MAXPARAMS+10);
n_tests_failed += check_errorcode( TESTLOG, 150, UNUR_ERR_DISTR_NPARAMS );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 153, (unur_ssr_chg_pdfparams(gen,fpar,-1)) );
n_tests_failed += check_errorcode( TESTLOG, 153, UNUR_ERR_DISTR_NPARAMS );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 156, (unur_ssr_chg_cdfatmode(gen,-1.)) );
n_tests_failed += check_errorcode( TESTLOG, 156, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 159, (unur_ssr_chg_pdfatmode(gen,-1.)) );
n_tests_failed += check_errorcode( TESTLOG, 159, UNUR_ERR_PAR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 162, (unur_ssr_chg_domain(gen,1.,-1.)) );
n_tests_failed += check_errorcode( TESTLOG, 162, UNUR_ERR_DISTR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 165, (unur_ssr_chg_pdfarea(gen,-1.)) );
n_tests_failed += check_errorcode( TESTLOG, 165, UNUR_ERR_DISTR_SET );
unur_free(gen);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = chg  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_chg() */

/*---------------------------------------------------------------------------*/

void test_init (void)
{
	/* start test */
	printf("[init "); fflush(stdout);
	fprintf(TESTLOG,"\n[init]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_PAR   *par = NULL;
   par = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 175, (unur_init( par )) );
n_tests_failed += check_errorcode( TESTLOG, 175, UNUR_ERR_NULL );
}

/* invalid data */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   distr = unur_distr_cont_new();
   unur_distr_cont_set_pdf(distr,pdf);
   unur_distr_cont_set_mode(distr,0.);
   unur_distr_cont_set_pdfarea(distr,1.);
   par = unur_ssr_new(distr); 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 185, (unur_init( par )) );
n_tests_failed += check_errorcode( TESTLOG, 185, UNUR_ERR_GEN_DATA );
unur_distr_free(distr);
}

/* data missing in distribution object */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   distr = unur_distr_cont_new();
   par = NULL; 

unur_errno = 0;
/* mode, pdfarea */
unur_distr_cont_set_pdf(distr,pdf);
par = unur_utdr_new( distr );
n_tests_failed += check_expected_NULL( TESTLOG, 195, (unur_init(par)) );
n_tests_failed += check_errorcode( TESTLOG, 195, UNUR_ERR_DISTR_REQUIRED );

unur_errno = 0;
/* pdfarea */
unur_distr_cont_set_mode(distr,1.);
par = unur_utdr_new( distr );
n_tests_failed += check_expected_NULL( TESTLOG, 201, (unur_init(par)) );
n_tests_failed += check_errorcode( TESTLOG, 201, UNUR_ERR_DISTR_REQUIRED );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = init  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_init() */

/*---------------------------------------------------------------------------*/

void test_reinit (void)
{
	/* start test */
	printf("[reinit "); fflush(stdout);
	fprintf(TESTLOG,"\n[reinit]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* exist */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   distr = unur_distr_normal(NULL,0);
   par = unur_ssr_new(distr);
   gen = unur_init( par ); 
abort_if_NULL(TESTLOG, 210,    gen );

unur_errno = 0;
n_tests_failed += check_expected_reinit( TESTLOG, 213, (unur_ssr_reinit( gen )) );
unur_free(gen);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = reinit  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_reinit() */

/*---------------------------------------------------------------------------*/

void test_sample (void)
{
	/* start test */
	printf("[sample "); fflush(stdout);
	fprintf(TESTLOG,"\n[sample]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* compare */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   double fpar[2] = {0.,1.};
   distr = unur_distr_normal(fpar,0); 
   par = NULL; 

unur_errno = 0;
/* default algorithm */
par = unur_ssr_new(distr);
n_tests_failed += compare_double_sequence_par_start( TESTLOG, 226, par, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
/* default algorithm - verifying mode */
par = unur_ssr_new(distr);
unur_ssr_set_verify(par,1);
n_tests_failed += compare_double_sequence_par( TESTLOG, 231, par, COMPARE_SAMPLE_SIZE );
unur_distr_free(distr);
}

/* compare */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   double fpar[2] = {0.,1.};
   distr = unur_distr_normal(fpar,2);
   par = NULL; 

unur_errno = 0;
/* use cdf at mode */
par = unur_ssr_new(distr);
unur_ssr_set_cdfatmode(par,0.5);
n_tests_failed += compare_double_sequence_par_start( TESTLOG, 243, par, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
/* use cdf at mode and squeeze */
par = unur_ssr_new(distr);
unur_ssr_set_cdfatmode(par,0.5);
unur_ssr_set_usesqueeze(par,1);
n_tests_failed += compare_double_sequence_par( TESTLOG, 249, par, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
/* use cdf at mode - verifying mode */
par = unur_ssr_new(distr);
unur_ssr_set_cdfatmode(par,0.5);
unur_ssr_set_verify(par,1);
n_tests_failed += compare_double_sequence_par( TESTLOG, 255, par, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
/* use cdf at mode and squeeze - verifying mode */
par = unur_ssr_new(distr);
unur_ssr_set_cdfatmode(par,0.5);
unur_ssr_set_usesqueeze(par,1);
unur_ssr_set_verify(par,1);
n_tests_failed += compare_double_sequence_par( TESTLOG, 262, par, COMPARE_SAMPLE_SIZE );
unur_distr_free(distr);
}

/* violate condition */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
   double fpar[2] = {0.,1.};
   distr = unur_distr_normal(fpar,2);
   par = NULL; 
/* pdf at mode is too small: hat < pdf near mode */

unur_errno = 0;
/* default algorithm */
par = unur_ssr_new(distr);
unur_ssr_set_pdfatmode(par,0.1);
 run_verify_generator ( TESTLOG,276, par );
n_tests_failed += check_errorcode( TESTLOG, 276, UNUR_ERR_GEN_CONDITION );

unur_errno = 0;
/* use cdf at mode and squeeze */
par = unur_ssr_new(distr);
unur_ssr_set_pdfatmode(par,0.1);
unur_ssr_set_cdfatmode(par,0.5);
unur_ssr_set_usesqueeze(par,1);
 run_verify_generator ( TESTLOG,283, par );
n_tests_failed += check_errorcode( TESTLOG, 283, UNUR_ERR_GEN_CONDITION );
/* pdf at mode is too large: squeeze > pdf near mode */

unur_errno = 0;
/* use cdf at mode and squeeze */
unur_distr_cont_set_pdfarea(distr,10.);
par = unur_ssr_new(distr);
unur_ssr_set_cdfatmode(par,0.5);
unur_ssr_set_pdfatmode(par,10.);
unur_ssr_set_usesqueeze(par,1);
 run_verify_generator ( TESTLOG,293, par );
n_tests_failed += check_errorcode( TESTLOG, 293, UNUR_ERR_GEN_CONDITION );
unur_distr_free(distr);
}

/* compare clone */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   UNUR_GEN *clone;
   distr = unur_distr_normal(NULL,0); 
   par = NULL;
   gen = NULL; 

unur_errno = 0;
/* original generator object */
par = unur_ssr_new(distr);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 306, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
/* clone */
clone = unur_gen_clone(gen);
unur_free(gen);
gen = clone;
n_tests_failed += compare_double_sequence_gen( TESTLOG, 312, gen, COMPARE_SAMPLE_SIZE );
unur_free(gen);
unur_distr_free(distr);
}

/* compare stringparser */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[2] = {3.,3.};
   distr = NULL;
   par = NULL;
   gen = NULL; 

unur_errno = 0;
distr = unur_distr_beta(fpar,2);
unur_distr_cont_set_pdfarea(distr,2);
par = unur_ssr_new(distr);
unur_ssr_set_cdfatmode(par,0.5);
unur_ssr_set_pdfatmode(par,1.9);
unur_ssr_set_usesqueeze(par,FALSE);
unur_ssr_set_verify(par,TRUE);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 330, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "beta(3.,3.); pdfarea = 2. & \
  method = ssr; cdfatmode = 0.5; usesqueeze = off; pdfatmode = 1.9; verify" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 336, gen, COMPARE_SAMPLE_SIZE );
unur_free(gen);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = sample  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_sample() */

/*---------------------------------------------------------------------------*/

void test_validate (void)
{
	UNUR_DISTR *distr[29];
	UNUR_PAR *par;
	UNUR_GEN *gen;
	int n_tests_failed = 0;
	double *darray;
	double fpm[10];

	/* start test */
	printf("[validate "); fflush(stdout);
	fprintf(TESTLOG,"\n[validate]\n");

	/* reset counter */
	n_tests_failed = 0;


/* distributions: 29 */
fpm[0] = 1.;
fpm[1] = 2.;
distr[0] = unur_distr_beta(fpm,2);


fpm[0] = 1.;
fpm[1] = 5.;
distr[1] = unur_distr_beta(fpm,2);


fpm[0] = 1.;
fpm[1] = 100.;
distr[2] = unur_distr_beta(fpm,2);


fpm[0] = 3.;
fpm[1] = 4.;
distr[3] = unur_distr_beta(fpm,2);


fpm[0] = 5.;
fpm[1] = 100.;
distr[4] = unur_distr_beta(fpm,2);


fpm[0] = 500.;
fpm[1] = 300.;
distr[5] = unur_distr_beta(fpm,2);


fpm[0] = 5.;
fpm[1] = 10.;
fpm[2] = -3.;
fpm[3] = 15.;
distr[6] = unur_distr_beta(fpm,4);


distr[7] = unur_distr_cauchy(NULL,0);


fpm[0] = 1.;
fpm[1] = 20.;
distr[8] = unur_distr_cauchy(fpm,2);


distr[23] = unur_distr_exponential(NULL,0);


fpm[0] = 30.;
fpm[1] = -5.;
distr[24] = unur_distr_exponential(fpm,2);


fpm[0] = 1.;
distr[9] = unur_distr_gamma(fpm,1);


fpm[0] = 2.;
distr[10] = unur_distr_gamma(fpm,1);


fpm[0] = 3.;
distr[11] = unur_distr_gamma(fpm,1);


fpm[0] = 10.;
distr[12] = unur_distr_gamma(fpm,1);


fpm[0] = 1000.;
distr[13] = unur_distr_gamma(fpm,1);


fpm[0] = 5.;
fpm[1] = 1000.;
distr[14] = unur_distr_gamma(fpm,2);


fpm[0] = 5.;
fpm[1] = 1.e-5;
distr[15] = unur_distr_gamma(fpm,2);


fpm[0] = 5.;
fpm[1] = 10.;
fpm[2] = 1000;
distr[16] = unur_distr_gamma(fpm,3);


distr[25] = unur_distr_laplace(NULL,0);


fpm[0] = -10.;
fpm[1] = 100.;
distr[26] = unur_distr_laplace(fpm,2);


distr[17] = unur_distr_normal(NULL,0);


fpm[0] = 1.;
fpm[1] = 1.e-5;
distr[18] = unur_distr_normal(fpm,2);


fpm[0] = 0.;
fpm[1] = 1.e+5;
distr[19] = unur_distr_normal(fpm,2);


distr[20] = unur_distr_uniform(NULL,0);


fpm[0] = 1.;
fpm[1] = 20.;
distr[21] = unur_distr_uniform(fpm,2);


distr[27] = unur_distr_cont_new();
unur_distr_cont_set_pdf(distr[27],pdf_sqrtlin);
unur_distr_cont_set_dpdf(distr[27],dpdf_sqrtlin);
unur_distr_cont_set_cdf(distr[27],cdf_sqrtlin);
unur_distr_set_name(distr[27],"sqrtlin");
unur_distr_cont_set_mode(distr[27],0.);
unur_distr_cont_set_pdfarea(distr[27],2.);


distr[28] = unur_distr_cont_new();
unur_distr_cont_set_pdf(distr[28],pdf_sqrtlinshft);
unur_distr_cont_set_dpdf(distr[28],dpdf_sqrtlinshft);
unur_distr_cont_set_cdf(distr[28],cdf_sqrtlinshft);
unur_distr_set_name(distr[28],"sqrtlin");
unur_distr_cont_set_mode(distr[28],1000.);
unur_distr_cont_set_pdfarea(distr[28],2.);


distr[22] = unur_distr_cauchy(NULL,0);
unur_distr_cont_set_domain(distr[22],0.1,1.);
unur_distr_cont_upd_mode(distr[22]);
unur_distr_cont_upd_pdfarea(distr[22]);


	printf("\n(chi^2) "); fflush(stdout);

/* chi^2 tests: 116 */

	unur_set_default_debug(~UNUR_DEBUG_SAMPLE);
	fprintf( TESTLOG,"\nChi^2 Test:\n");

/* distribution [0] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[0]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[0]), distr[0] );
par = unur_ssr_new(distr[0]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[0]);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[0]);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [1] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[1]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[1]), distr[1] );
par = unur_ssr_new(distr[1]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[1]);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[1]);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [2] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[2]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[2]), distr[2] );
par = unur_ssr_new(distr[2]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

/* distribution [3] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[3]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[3]), distr[3] );
par = unur_ssr_new(distr[3]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[3]);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [4] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[4]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[4]), distr[4] );
par = unur_ssr_new(distr[4]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

/* distribution [5] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[5]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[5]), distr[5] );
par = unur_ssr_new(distr[5]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

/* distribution [6] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[6]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[6]), distr[6] );
par = unur_ssr_new(distr[6]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[6]);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [23] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[23]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[23]), distr[23] );
par = unur_ssr_new(distr[23]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[23]);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[23]);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [24] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[24]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[24]), distr[24] );
par = unur_ssr_new(distr[24]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[24]);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[24]);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [7] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[7]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[7]), distr[7] );
par = unur_ssr_new(distr[7]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[7]);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[7]);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [8] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[8]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[8]), distr[8] );
par = unur_ssr_new(distr[8]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[8]);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[8]);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [9] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[9]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[9]), distr[9] );
par = unur_ssr_new(distr[9]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[9]);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[9]);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [0] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[0]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[0]), distr[0] );
par = unur_ssr_new(distr[0]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[0]);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[0]);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [11] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[11]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[11]), distr[11] );
par = unur_ssr_new(distr[11]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[11]);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [12] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[12]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[12]), distr[12] );
par = unur_ssr_new(distr[12]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[12]);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [13] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[13]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[13]), distr[13] );
par = unur_ssr_new(distr[13]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

/* distribution [14] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[14]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[14]), distr[14] );
par = unur_ssr_new(distr[14]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[14]);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [15] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[15]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[15]), distr[15] );
par = unur_ssr_new(distr[15]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

/* distribution [16] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[16]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[16]), distr[16] );
par = unur_ssr_new(distr[16]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[16]);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '-' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

/* distribution [25] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[25]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[25]), distr[25] );
par = unur_ssr_new(distr[25]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[25]);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[25]);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [26] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[26]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[26]), distr[26] );
par = unur_ssr_new(distr[26]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[26]);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[26]);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [17] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[17]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[17]), distr[17] );
par = unur_ssr_new(distr[17]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[17]);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[17]);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [18] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[18]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[18]), distr[18] );
par = unur_ssr_new(distr[18]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[18]);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [19] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[19]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[19]), distr[19] );
par = unur_ssr_new(distr[19]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

/* distribution [20] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[20]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[20]), distr[20] );
par = unur_ssr_new(distr[20]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[20]);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[20]);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [21] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[21]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[21]), distr[21] );
par = unur_ssr_new(distr[21]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[21]);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '-' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[21]);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [22] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[22]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[22]);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[22]);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

/* distribution [27] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[27]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[27]), distr[27] );
par = unur_ssr_new(distr[27]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

/* distribution [28] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[28]);
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[28]), distr[28] );
par = unur_ssr_new(distr[28]);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	n_tests_failed += run_validate_chi2( TESTLOG, 0, gen, '+' );
	unur_free(gen);
	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

	printf("\n(verify hat) "); fflush(stdout);

/* verify hat tests: 116 */

	unur_set_default_debug(~UNUR_DEBUG_SAMPLE);
	fprintf( TESTLOG,"\nVerify Hat Test (squeeze <= PDF <= hat):\n");

/* distribution [0] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[0]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[0]), distr[0] );
par = unur_ssr_new(distr[0]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[0]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[0]);
	unur_ssr_set_pedantic(par,0);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [1] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[1]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[1]), distr[1] );
par = unur_ssr_new(distr[1]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[1]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[1]);
	unur_ssr_set_pedantic(par,0);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [2] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[2]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[2]), distr[2] );
par = unur_ssr_new(distr[2]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

/* distribution [3] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[3]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[3]), distr[3] );
par = unur_ssr_new(distr[3]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[3]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [4] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[4]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[4]), distr[4] );
par = unur_ssr_new(distr[4]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

/* distribution [5] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[5]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[5]), distr[5] );
par = unur_ssr_new(distr[5]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

/* distribution [6] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[6]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[6]), distr[6] );
par = unur_ssr_new(distr[6]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[6]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [23] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[23]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[23]), distr[23] );
par = unur_ssr_new(distr[23]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[23]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[23]);
	unur_ssr_set_pedantic(par,0);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [24] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[24]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[24]), distr[24] );
par = unur_ssr_new(distr[24]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[24]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[24]);
	unur_ssr_set_pedantic(par,0);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [7] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[7]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[7]), distr[7] );
par = unur_ssr_new(distr[7]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[7]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[7]);
	unur_ssr_set_pedantic(par,0);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [8] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[8]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[8]), distr[8] );
par = unur_ssr_new(distr[8]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[8]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[8]);
	unur_ssr_set_pedantic(par,0);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [9] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[9]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[9]), distr[9] );
par = unur_ssr_new(distr[9]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[9]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[9]);
	unur_ssr_set_pedantic(par,0);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [0] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[0]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[0]), distr[0] );
par = unur_ssr_new(distr[0]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[0]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[0]);
	unur_ssr_set_pedantic(par,0);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [11] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[11]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[11]), distr[11] );
par = unur_ssr_new(distr[11]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[11]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [12] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[12]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[12]), distr[12] );
par = unur_ssr_new(distr[12]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[12]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [13] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[13]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[13]), distr[13] );
par = unur_ssr_new(distr[13]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

/* distribution [14] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[14]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[14]), distr[14] );
par = unur_ssr_new(distr[14]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[14]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [15] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[15]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[15]), distr[15] );
par = unur_ssr_new(distr[15]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

/* distribution [16] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[16]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[16]), distr[16] );
par = unur_ssr_new(distr[16]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

/* distribution [25] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[25]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[25]), distr[25] );
par = unur_ssr_new(distr[25]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[25]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[25]);
	unur_ssr_set_pedantic(par,0);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [26] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[26]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[26]), distr[26] );
par = unur_ssr_new(distr[26]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[26]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[26]);
	unur_ssr_set_pedantic(par,0);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [17] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[17]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[17]), distr[17] );
par = unur_ssr_new(distr[17]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[17]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[17]);
	unur_ssr_set_pedantic(par,0);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [18] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[18]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[18]), distr[18] );
par = unur_ssr_new(distr[18]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[18]);
	unur_ssr_set_pedantic(par,0);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [19] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[19]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[19]), distr[19] );
par = unur_ssr_new(distr[19]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

/* distribution [20] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[20]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[20]), distr[20] );
par = unur_ssr_new(distr[20]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[20]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

/* distribution [21] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[21]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[21]), distr[21] );
par = unur_ssr_new(distr[21]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[21]);
	unur_ssr_set_pedantic(par,0);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [22] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[22]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[22]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_domain(gen,0.9,0.92);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[22]);
	unur_ssr_set_pedantic(par,0);
fpm[0] = 1.;
fpm[1] = 4.;
	gen = unur_init(par);
	if (gen) {
unur_ssr_chg_pdfparams(gen,fpm,2);
unur_ssr_upd_pdfarea(gen);
unur_ssr_upd_mode(gen);
unur_ssr_reinit(gen);	
	}
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

/* distribution [27] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[27]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[27]), distr[27] );
par = unur_ssr_new(distr[27]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);

/* distribution [28] */

	do {
	unur_errno = 0;
par = unur_ssr_new(distr[28]);
	unur_ssr_set_pedantic(par,0);
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	do {
	unur_errno = 0;
{ double cdfatmode = unur_distr_cont_eval_cdf( unur_distr_cont_get_mode(distr[28]), distr[28] );
par = unur_ssr_new(distr[28]);
	unur_ssr_set_pedantic(par,0);
unur_ssr_set_cdfatmode(par,cdfatmode);
unur_ssr_set_usesqueeze(par,1); }
	gen = unur_init(par);
	if (gen) unur_ssr_chg_verify(gen,1);
	n_tests_failed += 10 * run_validate_verifyhat( TESTLOG, 0, gen, '~' );
	unur_free(gen);

	} while (0);

	printf("."); fflush(stdout);

	printf("."); fflush(stdout);


/* free distributions */
	unur_distr_free(distr[0]);
	unur_distr_free(distr[1]);
	unur_distr_free(distr[2]);
	unur_distr_free(distr[3]);
	unur_distr_free(distr[4]);
	unur_distr_free(distr[5]);
	unur_distr_free(distr[6]);
	unur_distr_free(distr[7]);
	unur_distr_free(distr[8]);
	unur_distr_free(distr[9]);
	unur_distr_free(distr[10]);
	unur_distr_free(distr[11]);
	unur_distr_free(distr[12]);
	unur_distr_free(distr[13]);
	unur_distr_free(distr[14]);
	unur_distr_free(distr[15]);
	unur_distr_free(distr[16]);
	unur_distr_free(distr[17]);
	unur_distr_free(distr[18]);
	unur_distr_free(distr[19]);
	unur_distr_free(distr[20]);
	unur_distr_free(distr[21]);
	unur_distr_free(distr[22]);
	unur_distr_free(distr[23]);
	unur_distr_free(distr[24]);
	unur_distr_free(distr[25]);
	unur_distr_free(distr[26]);
	unur_distr_free(distr[27]);
	unur_distr_free(distr[28]);
#if WITH_DMALLOC
	dmalloc_vmessage("section = validate  ================================\n","");
	dmalloc_log_unfreed();
#endif


	/* test finished */
	test_ok &= (n_tests_failed>2) ? 0 : 1;   /* we accept 2 failure */
	(n_tests_failed>2) ? printf(" --> failed] ") : printf(" --> ok] ");

	/* prevent compiler from making useless annoying warnings */
	distr[0] = NULL;
	par = NULL;
	gen = NULL;
	darray = NULL;
	fpm[0] = 0.;

} /* end of test_validate */

/*---------------------------------------------------------------------------*/


/* pdf that does not work */
double pdf( double x, const UNUR_DISTR *distr )
{ 
  return ((x==0.) ? 0. : exp(-x*x/2.));
} /* end of pdf */

/* pdf with piecewise linear function as transformed density with T = -1/sqrt */
double pdf_sqrtlin( double x, const UNUR_DISTR *distr )
{ 
	double y = 1./(fabs(x)+1.);
	return y*y;
}
double dpdf_sqrtlin( double x, const UNUR_DISTR *distr )
{ 
	double y = 1./(fabs(x)+1.);
	y = 2.*y*y*y;
	return ((x<0.) ? y : - y);
}
double cdf_sqrtlin( double x, const UNUR_DISTR *distr )
{ 
	if (x<=0.)
		return 0.5/(1.-x);
	else
		return (1.-0.5/(1.+x));
}

/* pdf with piecewise linear function as transformed density with T = -1/sqrt and shifted mode */
double pdf_sqrtlinshft( double x, const UNUR_DISTR *distr )
{ 
	double y;
	x -= 1000.;
	y = 1./(fabs(x)+1.);
	return y*y;
}
double dpdf_sqrtlinshft( double x, const UNUR_DISTR *distr )
{ 
	double y;
	x -= 1000.;
	y = 1./(fabs(x)+1.);
	y = 2.*y*y*y;
	return ((x<0.) ? y : - y);
}
double cdf_sqrtlinshft( double x, const UNUR_DISTR *distr )
{ 
	x -= 1000.;
	if (x<=0.)
		return 0.5/(1.-x);
	else
		return (1.-0.5/(1.+x));
}

/* dummy function */
int unur_ssr_set_pedantic( struct unur_par *par, int pedantic )
{ return 1; }



/*---------------------------------------------------------------------------*/
/* run generator in verifying mode */

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par )
{
	UNUR_GEN *gen;
	int i;

	/* switch to verifying mode */
	unur_ssr_set_verify(par,1);

	/* initialize generator */
	gen = unur_init( par ); abort_if_NULL(LOG, line, gen);

	/* run generator */
	for (i=0; i<VIOLATE_SAMPLE_SIZE; i++)
		unur_sample_cont(gen);

	/* destroy generator */
	unur_free(gen); 

} /* end of run_verify_generator() */

